@@ -4,7 +4,7 @@ rate_con_schema = {
     "transaction_type": [str],
     "sender": [str, type(None)],  # BROKER or SHIPPER if they are not having a broker. Example: "Werner Logistics"
     "receiver": [dict],
-    "client": [str, type(None)],  # hard-coded. Example: "Werner Logistics"
+    "client": [str, type(None)],   # hard-coded. Example: "Werner Logistics"
     "submitted_time": [str, type(None)],  # time we received the email
     "identifier": [str, type(None)],
     "identifier_type": [str, type(None)],
@@ -43,12 +43,13 @@ shipment_schema = {
     "weight": [str, type(None)],
     "weight_unit_code": [str, type(None)],
     "weight_qualifier": [str],  # hard-coded
-    "volume": [str, type(None)],
+    "volume": [str],
+    "distance": [str],
     "volume_qualifier": [str, type(None)],
     "truck_type": [str, type(None)],
     "temperature": [str, type(None)],
     "trucklength": [str, type(None)],
-    "charges": [str, type(None)],
+    "charges": [str, int],
     "loading_quantity": [str, type(None)]
 }
 
@@ -64,21 +65,21 @@ stop_schema = {
 }
 
 note_schema = {
-    "note": [str, type(None)],
-    "notetype": [str, type(None)],
-    "_notetype": [str, type(None)]
+    "note": [str],
+    "notetype": [str],
+    "_notetype": [str]
 }
 
 reference_schema = {
-    "id": [str, type(None)],
-    "idtype": [str, type(None)],
-    "_idtype": [str, type(None)]
+    "id": [str],
+    "idtype": [str],
+    "_idtype": [str]
 }
 
 dates_schema = {
-    "date": [str, type(None)],  # dd/mm/yyyy hh:mm
-    "datetype": [str, type(None)],  # always "RESPOND BY", "EP", or "LP"?None
-    "time": [str, type(None)],
+    "date": [str],  # yyyy-mm-dd
+    "datetype": [str],  # always "RESPOND BY", "EP", or "LP"?None
+    "time": [str, type(None)],  # hh:mm
     "timetype": [str, type(None)]
     # always "MUST RESPOND BY", "EARLIEST REQUESTED (PICKUP|DROP) TIME", "LATEST REQUESTED (PICKUP|DROP) TIME"?
 }
@@ -109,23 +110,27 @@ def schema_keys_validator(schema, actual_dict, key_type, only_warnining=True):
         schema_keys = list(schema.keys())
         actual_keys = list(actual_dict.keys())
         matched_keys = list()
+        keys_not_found = list()
         for key in schema_keys:
             if key in actual_keys:
-                actual_keys.remove(key)
                 matched_keys.append(key)
+                pass
+            else:
+                keys_not_found.append(key)
 
-        if len(actual_keys) > 0:
-            print(Fore.RED + f"[FAILED][{key_type} KEYS NOT FOUND]:{actual_keys}" + Fore.BLACK)
+        if len(keys_not_found) > 0:
+            print(Fore.RED + f"[FAILED][{key_type} KEYS NOT FOUND]:{keys_not_found}" + Fore.BLACK)
             schema_check_passed = False
-            return actual_keys, matched_keys, False
+            return keys_not_found, matched_keys, False
         else:
             if not only_warnining:
                 print(Fore.GREEN + f'[PASSED][ALL {key_type} KEY FOUND]' + Fore.BLACK)
-            return actual_keys, matched_keys, True
+            return keys_not_found, matched_keys, True
 
     except Exception as e:
         schema_check_passed = False
         print(Fore.RED + f"[FAILED][SCHEMA-CHECK][schema_keys_validator][{e}][{Exception}]" + Fore.BLACK)
+        return [], [], schema_check_passed
 
 
 def schema_key_type_validator(schema, actual_dict, key_type, only_warnining=True):
@@ -137,7 +142,6 @@ def schema_key_type_validator(schema, actual_dict, key_type, only_warnining=True
             for value in schema[key]:
                 if isinstance(actual_dict[key], value):
                     matched_type_keys.append(key)
-                    #                 print(f"{key}: PASSED")
                     break
             else:
                 schema_check.append(key)
@@ -153,11 +157,12 @@ def schema_key_type_validator(schema, actual_dict, key_type, only_warnining=True
     except Exception as e:
         schema_check_passed = False
         print(Fore.RED + f"[FAILED][SCHEMA-CHECK][schema_key_type_validator][{e}][{Exception}]" + Fore.BLACK)
+        return [], [], schema_check_passed
 
 
 def rate_con_schema_validation(actual_rate_con, rate_con_schema, stop_schema, entity_schema, note_schema, reference_schema,
-                        purchase_order_schema, contact_schema, dates_schema, shipment_schema, receiver_schema,
-                        only_warning=True):
+                               purchase_order_schema, contact_schema, dates_schema, shipment_schema, receiver_schema,
+                               only_warning=True):
     global schema_check_passed
     try:
         base_keys, matched_keys, success = schema_keys_validator(rate_con_schema, actual_rate_con, 'RATECON',
@@ -166,21 +171,20 @@ def rate_con_schema_validation(actual_rate_con, rate_con_schema, stop_schema, en
             base_keys, base_schema_matched_type_keys, success = schema_key_type_validator(rate_con_schema,
                                                                                           actual_rate_con, 'RATECON',
                                                                                           only_warning)
-            base_keys, matched_keys, success = schema_keys_validator(shipment_schema, actual_rate_con['shipment'],
-                                                                     'SHIPMENT', only_warning)
-
-            if 'shipment' in base_schema_matched_type_keys:
-                base_keys, shipment_schema_matched_type_keys, success = schema_key_type_validator(shipment_schema,
-                                                                                                  actual_rate_con[
+            base_keys, matched_keys, shipment_schema_success = schema_keys_validator(shipment_schema, actual_rate_con['shipment'], 'SHIPMENT', only_warning)
+            if shipment_schema_success:
+                if 'shipment' in base_schema_matched_type_keys:
+                    base_keys, shipment_schema_matched_type_keys, success = schema_key_type_validator(shipment_schema,
+                                                                                                      actual_rate_con[
                                                                                                       'shipment'],
-                                                                                                  'SHIPMENT',
-                                                                                                  only_warning)
+                                                                                                      'SHIPMENT',
+                                                                                                      only_warning)
 
             if 'receiver' in base_schema_matched_type_keys:
                 base_keys, matched_keys, receiver_success = schema_keys_validator(receiver_schema,
                                                                                   actual_rate_con['receiver'],
                                                                                   'RECEIVER', only_warning)
-                if success:
+                if receiver_success:
                     base_keys, receiver_schema_matched_type_keys, success = schema_key_type_validator(receiver_schema,
                                                                                                       actual_rate_con[
                                                                                                           'receiver'],
@@ -349,7 +353,7 @@ def rate_con_schema_validation(actual_rate_con, rate_con_schema, stop_schema, en
                                                                                               only_warning)
                     else:
                         schema_check_passed = False
-                        print(Fore.RED + f'[FAILED][DATES][{type(note)}TYPE WRONG]' + Fore.BLACK)
+                        print(Fore.RED + f'[FAILED][NOTES][{type(note)}TYPE WRONG]' + Fore.BLACK)
     except Exception as e:
         schema_check_passed = False
         print(Fore.RED + f"[FAILED][SCHEMA-CHECK][rate_con_validation][{e}][{Exception}]" + Fore.BLACK)
